#ifndef FILECACHE_H_
#define FILECACHE_H_

#include <map>
#include <string>
#include <vector>

#include "FileStructure.h"

class Sequence;

class FileCache
{
	public:
		FileCache(const std::string& aBaseDir);
		FileCache(const FileCache& anOther);
		virtual ~FileCache();

		FileCache& operator=(const FileCache& anOther);

		void generate(const std::string& aDirectory, const std::string& aFilename);
		FileStructure* update(const std::string& aDirectory, const std::string& aFilename);

		void addIncludeDir(const std::string& aBaseDir, const std::string& aDir);
		void addIncludeDirFromFile(const std::string& aBaseDir, const std::string& aFilename);
		std::string getAllDependenciesLine() const;
	
		void SetObjectsShouldContainDirectories() { ObjectsContainDirectories=true; };
		void SetObjectsShouldNotContainDirectories() { ObjectsContainDirectories=false; };
	
		void SetCompileCommand( const std::string &cmd );
		void SetQuiteModeOn() {QuiteMode = true; }
		void SetQuiteModeOff() {QuiteMode = false; }
	private:
		typedef std::map<std::string, FileStructure*> FileMap;
		FileMap Files;
		std::vector<std::string> IncludeDirs;
		std::vector<std::string> AllDependencies;
		std::string BaseDir;
		std::string CompileCommand;
		bool ObjectsContainDirectories;
		static bool QuiteMode;
};

#endif

