#include <unistd.h>
#include <getopt.h>
#include <iostream>

#include "FileCache.h"
#include "FileStructure.h"

#define CWDBUFFERLENGTH 1024

void PrintVersion()
{
	std::cout	<< "fastdep v0.09" << std::endl
					<< "copyright Bart Vanhauwaert, 2001, 2002" << std::endl
					<< "licensed under the GNU Public License" << std::endl;
}


/*
 * Generate dependencies for all files.
 */
int main(int argc, char **argv)
{
	char cwd_buffer[CWDBUFFERLENGTH+1];
	cwd_buffer[0] = 0;
	getcwd(cwd_buffer, CWDBUFFERLENGTH);
	FileCache Cache(cwd_buffer);
	Cache.addIncludeDir(cwd_buffer,".");
	std::string RemakeDepTarget;
	std::vector<std::string> ExtraRemakeDeps;

	while (1)
	{
		int option_index = 0;
		static struct option long_options[] =
		{
			{"remakedeptarget", 1, 0, 0 },
			{"extraremakedep", 1, 0, 0 },
			{"version", 0, 0, 0 },
			{"stdinclude", 2, 0, 0 },
			{"compile", 1, 0, 0 },
            {"extraincludes", 1, 0, 0},
			{0, 0, 0, 0}
		};
		int c = getopt_long(argc,argv,"vI:dq",long_options, &option_index);
		if (c==-1)
			break;
		switch (c)
		{
			case 0:
				switch (option_index)
				{
					case 0:
						RemakeDepTarget = std::string(optarg);
						break;
					case 1:
						ExtraRemakeDeps.push_back(std::string(optarg));
						break;
					case 2:
						PrintVersion();
						return 0;
				        case 3:
					  if (optarg)
					    {
					      Cache.addIncludeDir(cwd_buffer, optarg);
					    }
					  else
					    {
					      Cache.addIncludeDir(cwd_buffer, "/usr/include");
					    }
					  break;
                   case 4:
					 Cache.SetCompileCommand( std::string(optarg) );
					 break;
                   case 5:
                     Cache.addIncludeDirFromFile(cwd_buffer, optarg);
                     break;
				}
				break;
            case 'I':
                Cache.addIncludeDir(cwd_buffer, optarg);
                break;
            case 'v':
                PrintVersion();
                return 0;
            case 'd':
                Cache.SetObjectsShouldNotContainDirectories();
                break;
            case 'q':
                Cache.SetQuiteModeOn();
		}
	}
	if (optind == argc)
	{
		std::cout << "Usage : fastdep filename.c" << std::endl;
		return 0;
	}
	for (int i=optind; i<argc; ++i)
	{
		Cache.generate(cwd_buffer,argv[i]);
	}
	if (RemakeDepTarget != "")
	{
		std::cout << RemakeDepTarget << ":" << Cache.getAllDependenciesLine();
		for (unsigned int i=0; i<ExtraRemakeDeps.size(); ++i)
			std::cout << " \\\n\t" << ExtraRemakeDeps[i];
		std::cout << std::endl;
	}

	return 0;
}
